@@ -25,12 +25,12 @@ from bigjob.state import Running, New, Failed, Done, Unknown
 
 # Optional Job Plugins
 try:
-    from .job_plugin.gcessh import Service as GCEService
+    from job_plugin.gcessh import Service as GCEService
 except:
     pass 
 
 try:
-    from .job_plugin.ec2ssh import Service as EC2Service
+    from job_plugin.ec2ssh import Service as EC2Service
 except:
     pass 
 
@@ -326,7 +326,7 @@ class bigjob(api.base.bigjob):
                                                                           # or another external scheduler
                                                           )
         logger.debug("Adaptor specific modifications: "  + str(lrms_saga_url.scheme))
-        if is_bliss:
+        if is_bliss and lrms_saga_url.scheme.startswith("condor")==False:
             bootstrap_script = self.__escape_bliss(bootstrap_script)
         else:
             if lrms_saga_url.scheme == "gram":
@@ -351,11 +351,14 @@ class bigjob(api.base.bigjob):
            
             jd.executable = "/usr/bin/env"
             jd.arguments = ["python",  os.path.basename(condor_bootstrap_filename)]                
+            if pilot_compute_description.has_key("candidate_hosts"):
+                jd.candidate_hosts = pilot_compute_description["candidate_hosts"]
             bj_file_transfers = []
             file_transfer_spec = condor_bootstrap_filename + " > " + os.path.basename(condor_bootstrap_filename)
             bj_file_transfers.append(file_transfer_spec)
             output_file_name = "output-" + str(self.uuid) + ".tar.gz"
-            output_file_transfer_spec = os.path.join(self.working_directory, output_file_name) +" < " + output_file_name
+            #output_file_transfer_spec = os.path.join(self.working_directory, output_file_name) +" < " + output_file_name
+            output_file_transfer_spec = output_file_name +" < " + output_file_name
             #output_file_transfer_spec = os.path.join(self.working_directory, "output.tar.gz") +" < output.tar.gz"
             logger.debug("Output transfer: " + output_file_transfer_spec)
             bj_file_transfers.append(output_file_transfer_spec)
@@ -604,12 +607,12 @@ except:
     os.system("/usr/bin/env")
     try:
         os.system("python " + BOOTSTRAP_FILE + " " + BIGJOB_PYTHON_DIR); 
-        activate_this = BIGJOB_PYTHON_DIR+'bin/activate_this.py'; 
+        activate_this = os.path.join(BIGJOB_PYTHON_DIR, "bin/activate_this.py"); 
         execfile(activate_this, dict(__file__=activate_this))
     except:
         print "BJ installation failed. Trying system-level python (/usr/bin/python)";
         os.system("/usr/bin/python " + BOOTSTRAP_FILE + " " + BIGJOB_PYTHON_DIR); 
-        activate_this = BIGJOB_PYTHON_DIR+'bin/activate_this.py'; 
+        activate_this = os.path.join(BIGJOB_PYTHON_DIR, "bin/activate_this.py"); 
         execfile(activate_this, dict(__file__=activate_this))
 #try to import BJ once again
 import bigjob.bigjob_agent
@@ -816,8 +819,9 @@ bigjob_agent = bigjob.bigjob_agent.bigjob_agent(args)
     def __stage_files(self, filetransfers, target_url):
         logger.debug("Stage: %s to %s"%(filetransfers, target_url))
         if filetransfers==None:
-            return       
-        self.__filemanager.create_remote_directory(target_url)
+            return
+        if self.__filemanager:
+            self.__filemanager.create_remote_directory(target_url)
         for i in filetransfers:
             source_file=i
             if i.find(">")>0:
@@ -828,7 +832,8 @@ bigjob_agent = bigjob.bigjob_agent.bigjob_agent(args)
             target_url_full = os.path.join(target_url, os.path.basename(source_file))
             logger.debug("Stage: %s to %s"%(source_file, target_url_full))
             #self.__third_party_transfer(source_file, target_url_full)
-            self.__filemanager.transfer(source_file, target_url_full)
+            if self.__filemanager:
+                self.__filemanager.transfer(source_file, target_url_full)
            
        
 
@@ -1100,4 +1105,4 @@ class description(bliss.saga.job.Description):
                                         accessor=self.__class__.input_data) 
         self._register_rw_vec_attribute(name="OutputData", 
                                         accessor=self.__class__.output_data) 
-        
\ No newline at end of file
+        
